@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -26,11 +26,12 @@ USA.
 
 ;;;; Procedures to read a file in raw pgm format into a picture
 
-(declare (usual-integrations))
+;(declare (usual-integrations))
 
 (define (pgm-file->picture filename)
   (call-with-input-file (standard-pathname filename "pgm")
     (lambda (port)
+      (port/set-coding port 'BINARY)
       (get-body port (get-header port)))))
 
 (define (standard-pathname filename type)
@@ -56,7 +57,7 @@ USA.
   (let ((delimiters (char-set #\newline)))
     (lambda (port)
       (let loop ()
-	(let ((line (read-string delimiters port)))
+	(let ((line (read-delimited-string delimiters port)))
 	  (if (eof-object? line)
 	      (error "EOF encountered when parsing line."))
 	  (read-char port)
@@ -86,7 +87,7 @@ USA.
 	    (flo:vector-set!
 	     nth-row
 	     m
-	     (exact->inexact (char->ascii (read-char port)))))))))
+	     (exact->inexact (char->integer (read-char port)))))))))
     (picture-set-data! pic data)
     pic))
 
