@@ -3,7 +3,7 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -68,11 +68,9 @@ USA.
    (format	(runtime format)	(initialize-package!)	"format")
    (mime-codec	(runtime mime-codec)	#f			"mime-codec")
    (ordered-vector (runtime ordered-vector) #f			"ordvec")
-   (rb-tree	(runtime rb-tree)	#f			"rbtree")
    (stepper	(runtime stepper)	#f			"ystep")
    (subprocess	(runtime subprocess)	(initialize-package!)	"process")
    (synchronous-subprocess (runtime synchronous-subprocess) #f	"syncproc")
-   (wt-tree	(runtime wt-tree)	#f			"wttree")
    ))
 
 (define-load-option 'regular-expression
@@ -90,4 +88,6 @@ USA.
 			  "rexp"))
 
 ;; HASH-TABLE is now always loaded.
-(define-load-option 'hash-table dummy-option-loader)
\ No newline at end of file
+(define-load-option 'hash-table dummy-option-loader)
+(define-load-option 'rb-tree dummy-option-loader)
+(define-load-option 'wt-tree dummy-option-loader)
\ No newline at end of file
